@@ -17,7 +17,7 @@ function noteElement (note, currentNote = false) {
       id: note.id,
       title: note.title,
       label: note.id,
-      color: currentNote ? 'white' : 'black',
+      color: 'white',
       bgColor: currentNote ? 'black' : 'gray'
     }
   }
@@ -26,32 +26,31 @@ function noteElement (note, currentNote = false) {
 function linkElement (type, source, target) {
   console.assert(['backlink', 'direct', 'sequence'].includes(type));
   const id = type.slice(0, 1) + `${source}-${target}`;
-  const style = type === 'backlink' ? 'dashed' : 'solid';
-  const color = type === 'sequence' ? 'red' : 'black';
+  const style = type === 'sequence' ? 'dashed' : 'solid';
+  const color = 'black';
   return {
     data: { id, source, target, arrow: 'triangle', style, color }
   }
 }
 
-function * neighborElements (query, id) {
-  const note = query.note(id);
+function * neighborElements (query, note) {
   yield noteElement(note, true);
 
   for (const backlink of note.backlinks()) {
     yield noteElement(backlink.src);
-    yield linkElement('backlink', backlink.src.id, id);
+    yield linkElement('backlink', backlink.src.id, note.id);
   }
   for (const link of note.links()) {
     yield noteElement(link.dest);
-    yield linkElement('direct', id, link.dest.id);
+    yield linkElement('direct', note.id, link.dest.id);
   }
   for (const parent of note.parents()) {
     yield noteElement(parent.note);
-    yield linkElement('sequence', parent.note.id, id);
+    yield linkElement('sequence', parent.note.id, note.id);
   }
   for (const child of note.children()) {
     yield noteElement(child.note);
-    yield linkElement('sequence', id, child.note.id);
+    yield linkElement('sequence', note.id, child.note.id);
   }
   for (const alias of note.aliases()) {
     for (const ancestor of query.ancestors(alias)) {
@@ -156,7 +155,11 @@ function init (query) {
     const id = Number(window.location.hash.slice(1));
     if (!Number.isInteger(id)) return
 
-    const elements = Array.from(neighborElements(query, id));
+    const note = query.note(id);
+    const elements = [];
+    if (note != null) {
+      elements.push(...neighborElements(query, note));
+    }
     if (elements.length < 2) return
 
     container = graphArea();
@@ -369,9 +372,7 @@ class Link {
       annotation: this.annotation
     };
     src.links.push(link);
-    if (link.annotation) {
-      dest.backlinks.push(link);
-    }
+    dest.backlinks.push(link);
   }
 }
 
@@ -429,8 +430,7 @@ class Query {
     const record = this.db.data.aliases[alias];
     if (!record || !record.parent) return null
     const parentRecord = this.db.data.aliases[record.parent];
-    // TODO what if parentRecord.id === 0?
-    if (!parentRecord || !parentRecord.id) return null
+    if (!parentRecord || parentRecord.id == null) return null
     return {
       note: this.note(parentRecord.id),
       alias: record.parent
@@ -517,13 +517,9 @@ function createFuse () {
   return new Fuse(sections, options)
 }
 
-function getSearchResults (fuse) {
-  const div = document.querySelector('input.search-bar');
-  return fuse.search(div.value)
-}
-
-function displayResults (results) {
-  const div = document.querySelector('div.search-results');
+function displayResults (results, container) {
+  // Display results in container.
+  const div = container;
   div.textContent = '';
   for (const result of results) {
     const p = document.createElement('p');
@@ -546,27 +542,24 @@ function displayResults (results) {
   }
 }
 
-function searchNotes (fuse) {
-  const results = getSearchResults(fuse);
-  displayResults(results);
-}
-
-function searchBar () {
-  const form = document.createElement('form');
-  form.action = 'javascript:void(0)';
-  form.style.textAlign = 'center';
-
-  const fuse = createFuse();
-
+function searchBar (fuse, container) {
+  // Container will contain search results.
   const input = document.createElement('input');
   input.type = 'text';
   input.placeholder = 'Search notes...';
   input.classList.add('search-bar');
-  input.style.width = '80%';
-  input.addEventListener('change', () => searchNotes(fuse));
+  input.style.width = '100%';
+  input.style.background = 'transparent';
+  input.style.border = 'none';
+  input.style.borderBottomStyle = 'solid';
+  input.style.borderWidth = 'thin';
 
-  form.appendChild(input);
-  return form
+  input.addEventListener('change', function () {
+    window.location.hash = '#search';
+    const results = fuse.search(input.value);
+    displayResults(results, container);
+  });
+  return input
 }
 
 function searchResults () {
@@ -575,47 +568,50 @@ function searchResults () {
   return div
 }
 
-function searchPage () {
+function searchPage (container) {
+  // Container will contain search results.
   const page = document.createElement('section');
   page.id = 'search';
   page.title = 'Search';
   page.classList.add('level1');
-  page.appendChild(searchBar());
-  page.appendChild(document.createElement('br'));
-  page.appendChild(searchResults());
+  page.appendChild(container);
   return page
 }
 
-function searchButton () {
-  const a = document.createElement('a');
-  a.innerHTML = '<svg xmlns="http://www.w3.org/2000/svg" width="24" height="24" viewBox="0 0 24 24"><path d="M23.809 21.646l-6.205-6.205c1.167-1.605 1.857-3.579 1.857-5.711 0-5.365-4.365-9.73-9.731-9.73-5.365 0-9.73 4.365-9.73 9.73 0 5.366 4.365 9.73 9.73 9.73 2.034 0 3.923-.627 5.487-1.698l6.238 6.238 2.354-2.354zm-20.955-11.916c0-3.792 3.085-6.877 6.877-6.877s6.877 3.085 6.877 6.877-3.085 6.877-6.877 6.877c-3.793 0-6.877-3.085-6.877-6.877z"/></svg>';
-  a.href = '#search';
-  a.title = 'Search notes';
-  return a
+function searchSpan () {
+  const span = document.createElement('span');
+  span.innerHTML = '<svg xmlns="http://www.w3.org/2000/svg" width="24" height="24" viewBox="0 0 24 24"><path d="M23.809 21.646l-6.205-6.205c1.167-1.605 1.857-3.579 1.857-5.711 0-5.365-4.365-9.73-9.731-9.73-5.365 0-9.73 4.365-9.73 9.73 0 5.366 4.365 9.73 9.73 9.73 2.034 0 3.923-.627 5.487-1.698l6.238 6.238 2.354-2.354zm-20.955-11.916c0-3.792 3.085-6.877 6.877-6.877s6.877 3.085 6.877 6.877-3.085 6.877-6.877 6.877c-3.793 0-6.877-3.085-6.877-6.877z"/></svg>';
+  span.firstChild.style.height = '1em';
+  return span
+}
+
+function searchForm (container) {
+  // Container will contain search results.
+  const button = searchSpan();
+  const input = searchBar(createFuse(), container);
+  input.style.display = 'none';
+
+  button.addEventListener('click', function () {
+    button.style.display = 'none';
+    input.style.display = '';
+    input.focus();
+  });
+
+  input.addEventListener('focusout', function () {
+    button.style.display = '';
+    input.style.display = 'none';
+  });
+
+  const div = document.createElement('div');
+  div.appendChild(button);
+  div.appendChild(input);
+  return div
 }
 
 function init$1 () {
-  document.body.appendChild(searchPage());
-  document.body.insertBefore(searchButton(), document.body.firstChild);
-}
-
-function hideSections () {
-  const sections = document.getElementsByTagName('section');
-  for (let i = 0; i < sections.length; i++) {
-    const section = sections[i];
-    if (section.classList.contains('level1')) {
-      switch (true) {
-        case Number.isInteger(Number(section.id)):
-        case section.id.charAt(0) === '#':
-        case section.id === 'references':
-        case section.id.slice(0, 4) === 'ref-':
-        case section.id === 'search':
-        case section.id === 'tags':
-          section.style.display = 'none';
-          break
-      }
-    }
-  }
+  const container = searchResults();
+  document.body.appendChild(searchPage(container));
+  document.body.insertBefore(searchForm(container), document.body.firstChild);
 }
 
 function getSectionFromHash (hash) {
@@ -650,6 +646,7 @@ function notFoundSection () {
   const section = document.createElement('section');
   section.id = '0';
   section.classList.add('level1');
+  section.style.display = 'none';
   const h1 = document.createElement('h1');
   h1.innerText = 'Note not found';
   section.appendChild(h1);
@@ -658,8 +655,6 @@ function notFoundSection () {
 
 function init$2 () {
   document.body.appendChild(notFoundSection());
-  hideSections();
-
   const changeSection = sectionChanger();
   changeSection();
   window.addEventListener('hashchange', changeSection);
@@ -670,6 +665,8 @@ window.query = new Query(new Database());
 window.Model = Model;
 
 window.initSlipbox = function () {
+  const title = document.getElementById('title-block-header');
+  if (title) { title.remove(); }
   init$1();
   init$2();
   init(window.query);
