@@ -1,4 +1,5 @@
-local links_m = require "filters/links"
+local log = require "filters/log"
+local utils = require "filters/utils"
 
 local SlipBox = {}
 function SlipBox:new()
@@ -6,82 +7,95 @@ function SlipBox:new()
   return setmetatable({
     notes = {},
     links = {},
-    backlinks = {},
     aliases = {},
-    children = {},
     tags = {},
     citations = {},
   }, self)
 end
 
 function SlipBox:save_citation(id, citation)
-  id = tonumber(id)
-  if id then
-    local citations = self.citations[id] or {}
-    citations[citation] = true
-    self.citations[id] = citations
-  end
+  -- Save citation from note id (number).
+  assert(type(id) == "number")
+  local citations = self.citations[id] or {}
+  citations[citation] = true
+  self.citations[id] = citations
 end
 
-function SlipBox:save_note(note)
-  if note.id then
-    self.notes[note.id] = {title = note.title}
+function SlipBox:save_note(id, title)
+  -- Save note into slipbox.
+  -- Return list of error messages if a note with the same ID already
+  -- exists.
+  assert(type(id) == "number")
+  assert(type(title) == "string")
+  assert(title ~= "")
+
+  local note = self.notes[id]
+  if note then
+    return {
+      string.format("Duplicate ID: %d.", id),
+      string.format("Could not insert note '%s'.", title),
+      string.format("Note '%s' already uses the ID.", note.title)
+    }
   end
+  self.notes[id] = {title = title}
 end
 
-local function parent_sequence(s)
-  local t, count = s:gsub('^(.-)%d+$', '%1')
-  if count ~= 0 then return t end
-  t, count = s:gsub('^(.-)%a+$', '%1')
-  if count ~= 0 then return t end
-end
+function SlipBox:save_alias(id, alias, owner)
+  -- Save alias record in the slipbox.
+  -- Return list of error messages if the alias is already used by a
+  -- different note.
+  assert(type(id) == "number")
+  assert(utils.is_valid_alias(alias))
+  assert(alias)
 
-function SlipBox:save_backlink(link)
-  assert(link ~= nil)
-  assert(link.tag == "direct" and link.src and link.dest)
-  assert(link.description)
-  if link.description ~= "" then
-    local backlinks = self.backlinks[link.dest] or {}
-    table.insert(backlinks, link)
-    self.backlinks[link.dest] = backlinks
+  local record = self.aliases[alias]
+  if record and record.id ~= id then
+    return {
+      string.format("Duplicate alias definition for '%s' used by note %d.",
+        alias, record.id),
+      string.format("It will not be used as an alias for note %d.", id),
+    }
   end
+
+  self.aliases[alias] = {
+    id = id,
+    owner = owner,
+  }
 end
 
 function SlipBox:save_sequence(link)
-  -- Set aliases and children tables (for sequence/folgezettel notes).
+  -- Set aliases table for sequence/folgezettel notes.
+  -- Return list of error messages if source of sequence link is not the
+  -- same as the root of the sequence.
+  -- Also return list of error messages if note aliases can't be defined.
   assert(link ~= nil)
-  assert(link.tag == "sequence" and link.alias and link.root)
-  assert(link.root ~= "" and link.alias ~= "")
+  assert(type(link.dest) == "number")
+  assert(link.tag == "sequence" and link.description)
+  assert(link.description ~= "")
 
-  if link.dest and link.alias then
-    self.aliases[link.alias] = {
-      id = link.dest,
-      owner = links_m.alias_root(link.alias),
+  local owner = utils.alias_root(link.description)
+  if owner ~= tostring(link.src) then
+    return {
+      string.format(
+        "Note %d defines a note alias (%s) with a different root.",
+        link.src,
+        link.description,
+        owner
+      ),
+      "The alias definition will be ignored."
     }
-    -- dest might not be in notes if it's not in the current set of input files
-    if self.notes[link.dest] then
-      local aliases = assert(self.notes[link.dest]).aliases or {}
-      table.insert(aliases, link.alias)
-      self.notes[link.dest].aliases = aliases
-    end
-    local parent = parent_sequence(link.alias)
-    if parent then
-      local children = self.children[parent] or {}
-      table.insert(children, link.alias)
-      self.children[parent] = children
-
-      if parent == tostring(link.src) then
-        local aliases = assert(self.notes[link.src]).aliases or {}
-        table.insert(aliases, tostring(link.src))
-        self.notes[link.src].aliases = aliases
-
-        self.aliases[tostring(link.src)] = {
-          id = link.src,
-          owner = links_m.alias_root(link.alias),
-        }
-      end
-    end
   end
+
+  local err = self:save_alias(link.dest, link.description, owner)
+  if err then return err end
+
+  local parent = utils.alias_parent(link.description)
+  if parent == tostring(link.src) then
+    err = self:save_alias(link.src, parent, owner)
+    if err then return err end
+  end
+
+  -- TODO does this work even if dest or parent are not going to be scanned?
 end
 
 function SlipBox:save_link(link)
@@ -91,23 +105,24 @@ function SlipBox:save_link(link)
       table.insert(links, link)
       self.links[link.src] = links
     end
-    if link.tag == "direct" then
-      if link.description and link.description ~= "" then
-        self:save_backlink(link)
+    if link.tag == "sequence" then
+      local err = self:save_sequence(link)
+      if err then
+        log.warning(err)
       end
-    elseif link.tag == "sequence" then
-      self:save_sequence(link)
     end
   end
 end
 
 function SlipBox:save_tag(id, tag)
   -- Insert id into self:tags[tag].
-  if id and tag ~= "" then
-    local tags = self.tags[tag] or {}
-    table.insert(tags, id)
-    self.tags[tag] = tags
-  end
+  assert(type(id) == "number")
+  assert(type(tag) == "string")
+  assert(tag ~= "")
+
+  local tags = self.tags[tag] or {}
+  table.insert(tags, id)
+  self.tags[tag] = tags
 end
 
 local function sqlite_string(s)
@@ -204,17 +219,25 @@ local function aliases_to_sql(aliases)
   return ""
 end
 
-local function sequences_to_sql(sequences)
+local function sequences_to_sql(aliases)
   local values = ""
-  for alias, children in pairs(sequences) do
-    local parent = sqlite_string(alias)
-    for _, v in ipairs(children) do
-      local child = sqlite_string(v)
-      local value = string.format("(%s, %s)", parent, child)
-      if values == "" then
-        values = values .. ' ' .. value
+
+  for alias, rec in pairs(aliases) do
+    local parent = utils.alias_parent(alias)
+    if parent then
+      if not aliases[parent] then
+        log.warning {
+          string.format("Missing note alias: '%s'.", parent),
+          string.format("Note %d with alias '%s' will be unreachable.", rec.id, alias),
+        }
       else
-        values = values .. ", " .. value
+        local child = sqlite_string(alias)
+        local value = string.format("(%s, %s)", sqlite_string(parent), child)
+        if values == "" then
+          values = values .. ' ' .. value
+        else
+          values = values .. ", " .. value
+        end
       end
     end
   end
@@ -285,12 +308,10 @@ function SlipBox:to_sql(filenames)
   -- : Table that maps note ids to filenames (see scan.parse_grep_output).
   return files_to_sql(filenames) .. notes_to_sql(self.notes, filenames) ..
     tags_to_sql(self.tags) .. links_to_sql(self.links) ..
-    aliases_to_sql(self.aliases) .. sequences_to_sql(self.children) ..
+    aliases_to_sql(self.aliases) .. sequences_to_sql(self.aliases) ..
     citations_to_sql(self.citations)
 end
 
 return {
   SlipBox = SlipBox,
-  -- private:
-  parent_sequence = parent_sequence,
 }
