@@ -1,26 +1,29 @@
 """Generate complete HTML containing all slipbox notes."""
 
-import os
+from pathlib import Path
 import shlex
+from sqlite3 import Connection
 import subprocess
+from typing import Iterable
 
 from .templates import Elem, render
-from .utils import make_temporary_file, write_lines
+from .utils import pandoc, temporary_directory
 
-DUMMY_MARKDOWN = r"""$\,$
-``` {.c style="display:none"}
-```"""
+DUMMY_MARKDOWN = r"""::: {style="display:none"}
+$\,$
+```c
+```
+:::"""
 
-def generate_active_htmls(conn):
+def generate_active_htmls(conn: Connection) -> Iterable[str]:
     """Get HTML stored in the database for active sections."""
     sql = """
         SELECT body FROM Html WHERE id IN (SELECT html FROM Sections)
             ORDER BY id DESC
     """
-    cur = conn.cursor()
-    return map(lambda row: row[0], cur.execute(sql))
+    return (body for body, in conn.execute(sql))
 
-def generate_data(conn):
+def generate_data(conn: Connection) -> Iterable[str]:
     """Generate slipbox data in javascript."""
     for nid, title in conn.execute("SELECT id, title FROM Notes ORDER BY id"):
         yield f"window.query.db.add(new Model.Note({nid}, {title!r}))"
@@ -35,12 +38,11 @@ def generate_data(conn):
     for prev, next_ in conn.execute(sql):
         yield f"window.query.db.add(new Model.Sequence({prev!r}, {next_!r}))"
 
-def generate_javascript(conn):
+def generate_javascript(conn: Connection) -> Iterable[str]:
     """Generate slipbox javascript code."""
     yield '<script type="module">'
-    basedir = os.path.dirname(__file__)
-    with open(os.path.join(basedir, "data/bundle.js")) as file:
-        yield file.read().strip()
+    bundle = Path(__file__).parent/"data"/"bundle.js"
+    yield bundle.read_text().strip()
     yield '</script>'
     yield '<script type="text/javascript">'
     yield "window.addEventListener('DOMContentLoaded', () => {"
@@ -49,7 +51,7 @@ def generate_javascript(conn):
     yield "})"
     yield "</script>"
 
-def create_bibliography(conn):
+def create_bibliography(conn: Connection) -> str:
     """Create bibliography HTML section from database entries."""
     sql = "SELECT key, text FROM Bibliography ORDER BY key"
     items = []
@@ -61,10 +63,11 @@ def create_bibliography(conn):
                    Elem("dl", *items),
                    id="references",
                    title="References",
+                   style="display:none",
                    **{"class": "level1"})
     return render(section)
 
-def create_tags(conn):
+def create_tags(conn: Connection) -> str:
     """Create HTML section that lists all tags."""
     rows = conn.execute("SELECT DISTINCT tag FROM Tags ORDER BY tag")
     tags = (row[0] for row in rows)
@@ -74,10 +77,33 @@ def create_tags(conn):
                    Elem("ul", *items),
                    id="tags",
                    title="Tags",
+                   style="display:none",
                    **{"class": "level1"})
     return render(section)
 
-def create_tag_page(conn, tag):
+def create_entrypoints(conn: Connection) -> str:
+    """Create HTML section for entrypoints.
+
+    This contains all notes that starts a sequence.
+    """
+    query = """
+        SELECT id, title FROM Aliases JOIN Notes USING (id)
+            WHERE CAST(id AS STRING) = alias
+                ORDER BY title
+    """
+    entrypoints = conn.execute(query)
+    items = (Elem("li", Elem("a", title, href=f"#{nid}"))
+             for nid, title in entrypoints)
+    section = Elem("section",
+                   Elem("h1", "Entrypoints"),
+                   Elem("ul", *items),
+                   id="entrypoints",
+                   title="Entrypoints",
+                   style="display:none",
+                   **{"class": "level1"})
+    return render(section)
+
+def create_tag_page(conn: Connection, tag: str) -> str:
     """Create HTML section that lists all notes with the tag."""
     sql = """
         SELECT id, title FROM Tags NATURAL JOIN Notes WHERE tag = ? ORDER BY id
@@ -91,16 +117,17 @@ def create_tag_page(conn, tag):
                    Elem("ul", *items),
                    id=tag,
                    title=tag,
+                   style="display:none",
                    **{"class": "level1"})
     return render(section)
 
-def create_tag_pages(conn):
+def create_tag_pages(conn: Connection) -> str:
     """Create all tag pages."""
     rows = conn.execute("SELECT DISTINCT tag FROM Tags ORDER BY tag")
     tags = (row[0] for row in rows)
     return '\n'.join(create_tag_page(conn, tag) for tag in tags)
 
-def create_reference_page(conn, reference):
+def create_reference_page(conn: Connection, reference: str) -> str:
     """Create HTML section that lists all notes that cite the reference."""
     sql = """
         SELECT note, title, text FROM Citations
@@ -120,30 +147,34 @@ def create_reference_page(conn, reference):
                    Elem("ul", *items),
                    id=reference,
                    title=reference,
+                   style="display:none",
                    **{"class": "level1"})
     return render(section)
 
-def create_reference_pages(conn):
+def create_reference_pages(conn: Connection) -> str:
     """Create all reference pages."""
     rows = conn.execute("SELECT key FROM Bibliography ORDER BY key")
     references = (row[0] for row in rows)
     return '\n'.join(create_reference_page(conn, ref) for ref in references)
 
-def generate_complete_html(conn, options):
+def generate_complete_html(conn: Connection, options: str) -> None:
     """Create final HTML file with javascript."""
-    with make_temporary_file(suffix=".md", text=True) as dummy,\
-            make_temporary_file() as script,\
-            make_temporary_file(suffix=".html", text=True) as html,\
-            make_temporary_file(suffix=".html", text=True) as extra:
-        write_lines(dummy, [DUMMY_MARKDOWN])
-        write_lines(script, generate_javascript(conn))
-        write_lines(html, generate_active_htmls(conn))
+    with temporary_directory() as tempdir:
+        script = tempdir/"script.js"
+        html = tempdir/"cached.html"
+        extra = tempdir/"extra.html"
+        dummy = tempdir/"Slipbox.md"
+        dummy.write_text(DUMMY_MARKDOWN)
+        script.write_text('\n'.join(generate_javascript(conn)))
+        html.write_text('\n'.join(generate_active_htmls(conn)))
         with open(extra, "w") as file:
             print(create_tag_pages(conn), file=file)
             print(create_tags(conn), file=file)
             print(create_reference_pages(conn), file=file)
             print(create_bibliography(conn), file=file)
-        cmd = "pandoc {dummy} -H {script} -B {html} -B {extra} --section-divs {options}".format(
-            dummy=shlex.quote(dummy), script=shlex.quote(script),
-            html=shlex.quote(html), options=options, extra=shlex.quote(extra))
+            print(create_entrypoints(conn), file=file)
+        cmd = "{pandoc} {dummy} -H{script} {title} -B{html} -B{extra} --section-divs {opts}".format(
+            pandoc=pandoc(), dummy=shlex.quote(str(dummy)), script=shlex.quote(str(script)),
+            html=shlex.quote(str(html)), opts=options, extra=shlex.quote(str(extra)),
+            title="--metadata title=Slipbox")
         subprocess.run(shlex.split(cmd), check=False)
